#if (defined _MSC_VER)
#include <windows.h>
#endif
#include <GL/gl.h>
#include <math.h>
#include <iostream>

#include "Arrow.h"
#include "Vector2f.h"

#define PI (4*(float)atan(1))

/*!
 */
Arrow::Arrow() :
  m_arrow_length(20), m_arrow_width(10)
{}

/*!
 */
Arrow::Arrow(float arrow_length, float arrow_width) :
  m_arrow_length(arrow_length), m_arrow_width(arrow_width)
{}

/*! Copy constructor
 */
Arrow::Arrow(const Arrow & arrow)
{
  m_arrow_length = arrow.m_arrow_length;
  m_arrow_width = arrow.m_arrow_width;
}
  
/*! Assignment operator
 */
Arrow & Arrow::operator=(const Arrow & arrow)
{
  m_arrow_length = arrow.m_arrow_length;
  m_arrow_width = arrow.m_arrow_width;
  return *this;
}

/*! Draw an arrow at a given position in a given direction. The arrow base
 * width and arrow length are provided as data members.
 * \param pos the arrow position
 * \param dir the arrow direction
 */
void Arrow::draw_arrow(const Vector2f & pos, const Vector2f & dir,
                       bool fill) const
{
  float angle = atan2f(dir[1], dir[0]) * 180.0f / PI;
  glPushMatrix();
  glTranslatef(pos[0], pos[1], 0.0f);
  glRotatef(angle, 0.0f, 0.0f, 1.0f);
  glBegin((fill) ? GL_TRIANGLES : GL_LINE_LOOP);
  glVertex2f(m_arrow_length, 0.0f);
  glVertex2f(0.0f, m_arrow_width * 0.5f);
  glVertex2f(0.0f, -m_arrow_width * 0.5f);
  glEnd();
  glPopMatrix();
}
