#ifndef AUX_SPACE_H
#define AUX_SPACE_H

#include "Viewport.h"
#include "Frustum.h"
#include "Agent.h"
#include "Reshape_viewport_events.h"
#include "Screen_poly.h"
#include "State.h"
#include "Mode.h"
#include "Orientation.h"

class Trans_poly;
class Translate_event;
class Rotate_event;
class Mode_event;
class Simulate_event;
class Collision_test_event;
class Reset_event;

class Aux_space : public Agent {
private:
  class Aux_space_state;

  typedef Screen_poly<Reshape_as_viewport_event>   Poly_as_sim;

  enum States {
    IDLE_STATE = 0,
    REF_DIR_STATE,
    ROBOT_FIND_VERTEX_STATE,
    OBSTACLE_FIND_VERTEX_STATE,
    NUM_STATES
  };

  bool m_simulate;
  Mode m_mode;
  
  // Simulation stuff:
  Arrow m_ref_dir_arrow;
  Vector2f m_ref_dir;
  float m_ref_dir_length;
  float m_ref_dir_arrow_length;
  float m_ref_dir_arrow_width;
  int m_robot_extreme_vertex_id;
  int m_obstacle_extreme_vertex_id;
  
  int m_state_id;
  Aux_space_state * m_states[NUM_STATES];

  RGB_color m_normal_color;

  Viewport m_viewport;
  Frustum m_frustum;

  float m_scale;

  Poly_as_sim m_rot_robot;
  
  float m_near;
  float m_far;

  Trans_poly * m_robot;
  Trans_poly * m_obstacle;

  bool m_reshaped;

  void compute_ref_dir();
  
public:
  /*! Parameterless constructor
   */
  Aux_space(void);

  /*! Destructor
   */
  virtual ~Aux_space(void);
  
  /*!
   */
  virtual void identify(void);

  /*!
   */
  virtual void handle(Reshape_ws_viewport_event * reshape_event);

  /*!
   */
  virtual void handle(Translate_event * translate_event);

  /*!
   */
  virtual void handle(Rotate_event * rotate_event);

  /*!
   */
  virtual void handle(Mode_event * mode_event);

  /*!
   */
  virtual void handle(Simulate_event * mode_event);

  /*!
   */
  virtual void handle(Reset_event * Reset_event);

  /*!
   */
  virtual void handle(Collision_test_event * collision_event);
  
  /*!
   */
  void set_robot(Trans_poly & robot) { m_robot = &robot; }

  /*!
   */
  void set_obstacle(Trans_poly & obstacle) { m_obstacle = &obstacle; }

  /*!
   */
  void init(void);
  
  /*!
   */
  void draw(void);

  /*!
   */
  void set_viewport(int vp_left, int vp_bottom, int vp_width, int vp_height);

  /*!
   */
  Poly_as_sim & get_rot_robot(void) { return m_rot_robot; }

  /*!
   */
  void set_state(int state_id);

private:
  /*! Generic Aux_space state:
   */
  class Aux_space_state : public State {
  protected:
    Aux_space * m_aux_space;
    int m_state_id;
    
  public:
    Aux_space_state(Aux_space * as, int state_id) :
      m_aux_space(as), m_state_id(state_id)
    {}
    Aux_space * get_aux_space(void) const { return m_aux_space; }
    void change_state(int state_id);
    virtual void draw() {}
  };

  /*! Idle state (or start state)
   */
  class Idle_state : public Aux_space_state {
  public:
    Idle_state(Aux_space * as, int id) : Aux_space_state(as, id) {}
    virtual void identify(void);
    virtual void draw();
    void on_entry(void);
    void on_exit(void);
  };

  /*! Calculate reference direction state
   */
  class Ref_dir_state : public Aux_space_state {
  private:
    float m_factor;
    float m_alpha;
    float m_delta;
    
  public:
    Ref_dir_state(Aux_space * as, int id) :
      Aux_space_state(as, id), m_factor(2), m_alpha(0), m_delta(0) {}
    virtual void identify(void);
    virtual void draw();
    void on_entry(void);
    void on_exit(void);
    void handle(Tick_event * Tick_event);
  };

  /*! Find vertex state
   */
  class Find_vertex_state : public Aux_space_state {
  private:
    Trans_poly * m_poly;
    Vector2f m_origin;
    int m_prev_normal_id;
    int m_curr_normal_id;
    Orientation m_prev_orientation;
    Orientation m_curr_orientation;

    unsigned int compute_next();
    
  public:
    Find_vertex_state(Aux_space * as, int id) : Aux_space_state(as, id) {}
    virtual void identify(void);
    virtual void draw();
    void on_entry(void);
    void on_exit(void);
    void handle(Tick_event * Tick_event);
  };

  friend class Idle_state;
  friend class Ref_dir_state;
  friend class Find_vertex_state;
};

#endif
