#ifndef CIRCULAR_RANGE_H
#define CIRCULAR_RANGE_H

#include <iostream>

class Circular_range {
private:
  bool flag;
  
  unsigned int m_num;

  unsigned int m_start;
  unsigned int m_end;
  unsigned int m_next;

public:
  /*! Parameterless constructor.
   */
  Circular_range() : flag(false), m_num(0), m_start(0), m_end(0), m_next(0) {}
  Circular_range(int num, int start, int end) :
    m_num(num), m_start(start), m_end(end), m_next(0)
  {
  }

  /*! Copy constructor.
   */
  Circular_range(const Circular_range & range) :
    m_num(range.m_num),
    m_start(range.m_start),
    m_end(range.m_end),
    m_next(range.m_next)
  {
  }
  
  /*! Returns the size of the circular range.
   */
  unsigned int get_size() const
  {
    unsigned int tmp = m_end;
    if (m_start >= tmp) tmp += m_num;
    return tmp - m_start;
  }

  /*! Computes the range middle-point, and assigns it to the m_next.
   * The range is circular. It starts at m_start and it ends at m_end. When
   * m_end is equal to m_start, it is assumed that the range spans the entire
   * circle of normals. m_start and m_end of a given range are intialized to
   * 0, and this is the only time where m_end is equal to m_start. During
   * exection the range never shrinks to contain less than 2 normals.
   * Aalternates between the even and the odd middle points of an odd-size
   * range. This trick does not only improve the distribution, but is
   * necessary to guarantee end condition in certain 2 cases.
   *
   *      /|\------1
   *     / | \
   *    /  |  \
   *   /   |   \
   *  4    3    2
   *
   * Case 1: 1 & 3 are blue, and 2 & 4 are red.
   * Case 2: 1 & 4 are blue, and 2 & 3 are red.
   */
  void compute_next()
  {
    m_next = m_start + (get_size() + ((flag) ? 1 : 0)) / 2;
    if (m_next >= m_num) m_next -= m_num;
    flag = !flag;
  }

  /*! Get data-member methods
   */
  unsigned int get_start() const { return m_start; }
  unsigned int get_end() const { return m_end; }
  unsigned int get_next() const { return m_next; }
  unsigned int get_num() const { return m_num; }

  /*! Set data-member methods
   */
  void set_start(unsigned int start) { m_start = start; }
  void set_end(unsigned int end) { m_end = end; }
  void set_next(unsigned int next) { m_next = next; }
  void set_num(unsigned int num) { m_num = num; }
  
  /*! Returns true, if m_end immediately follows m_start, and false otherwise.
   */
  bool is_successive() const
  {
    return (((m_start + 1) % m_num) == m_end);
  }
};

/*!
 */
inline std::ostream & operator<<(std::ostream & os,
                                 const Circular_range & range)
{
  os << range.get_num()
     << "," << range.get_start() << "," << range.get_end()
     << "," << range.get_next();
  return os;
}  

#endif
