#include "Circular_search.h"

/*!
 */
bool Circular_search::operator()(const Vector2f & dir)
{
  // Initialize orientations:
  Vector2f origin;
  Vector2f start_normal;
  get_normal(m_range.get_start(), start_normal);
  m_start_orientation =
    start_normal.orientation(start_normal, origin, dir);

  Vector2f end_normal;
  get_normal(m_range.get_end(), end_normal);
  m_end_orientation =
    end_normal.orientation(end_normal, origin, dir);

  // Check if the reference is out of range:
  if (((m_start_orientation != LEFT_TURN) &&
       (m_end_orientation == LEFT_TURN)) ||
      ((m_start_orientation == m_end_orientation) &&
       (start_normal.orientation(start_normal, origin, end_normal) ==
        LEFT_TURN)))
    return false;
  
  // The ids are ordered clockwise:
  while (!m_range.is_successive()) {
    // Compute the id of the next normal and the extract the normal:
    m_range.compute_next();
    Vector2f next_normal;
    get_normal(m_range.get_next(), next_normal);
    m_next_orientation = next_normal.orientation(next_normal, origin, dir);
  
    // Set the ids to be clockwise ordered:
    if ((m_next_orientation == LEFT_TURN) &&
        (m_end_orientation != LEFT_TURN))
    {
      m_range.set_start(m_range.get_next());
      m_start_orientation = m_next_orientation;
    } else if ((m_next_orientation != LEFT_TURN) &&
               (m_start_orientation == LEFT_TURN))
    {
      m_range.set_end(m_range.get_next());
      m_end_orientation = m_next_orientation;
    } else {
      Vector2f end_normal;
      get_normal(m_range.get_end(), end_normal);
      Orientation orient =
        next_normal.orientation(next_normal, origin, end_normal);
      if (orient == RIGHT_TURN) {
        m_range.set_start(m_range.get_next());
        m_start_orientation = m_next_orientation;
      } else {
        m_range.set_end(m_range.get_next());
        m_end_orientation = m_next_orientation;
      }
    }
  }
  return true;
}
