#ifndef CIRCULAR_SEARCH_H
#define CIRCULAR_SEARCH_H

#include "Circular_range.h"
#include "Orientation.h"
#include "Vector2f.h"

class Circular_search {
private:
  virtual void get_normal(unsigned int id, Vector2f & normal) const = 0;
  virtual void get_vertex(unsigned int id, Vector2f & vertex) const = 0;
  virtual unsigned int get_num_normals() const = 0;
  virtual void draw(float radius) {}
  
protected:
  Orientation m_start_orientation;
  Orientation m_end_orientation;
  Orientation m_next_orientation;

  Circular_range m_range;
    
public:
  Circular_search() :
    m_start_orientation(LEFT_TURN),
    m_end_orientation(LEFT_TURN),
    m_next_orientation(LEFT_TURN)
  {
  }
  
  /*!
   */
  Circular_search(const Circular_search & clone) :
    m_start_orientation(clone.m_start_orientation),
    m_end_orientation(clone.m_end_orientation),
    m_next_orientation(clone.m_next_orientation),
    m_range(clone.m_range)
  {
  }

  /*!
   */
  Circular_search & operator=(const Circular_search & clone)
  {
    m_start_orientation = clone.m_start_orientation;
    m_end_orientation = clone.m_end_orientation;
    m_next_orientation = clone.m_next_orientation;
    m_range = clone.m_range;
    return *this;
  }
  
  /*!
   */
  void compute_next() { m_range.compute_next(); }

  /*!
   */
  bool is_successive() const { return m_range.is_successive(); }
  
  /*!
   */
  const Circular_range & get_range() const { return m_range; }
  
  /*!
   */
  unsigned int get_start() const { return m_range.get_start(); }
  unsigned int get_end() const { return m_range.get_end(); }
  unsigned int get_next() const { return m_range.get_next(); }

  /*!
   */
  void set_start(unsigned int id) { m_range.set_start(id); }
  void set_end(unsigned int id) { m_range.set_end(id); }
  void set_next(unsigned int id) { m_range.set_next(id); }

  /*!
   */
  Orientation get_start_orientation() const { return m_start_orientation; }
  Orientation get_end_orientation() const { return m_end_orientation; }
  Orientation get_next_orientation() const { return m_next_orientation; }

  /*!
   */
  void set_start_orientation(Orientation o) { m_start_orientation = o; }
  void set_end_orientation(Orientation o) { m_end_orientation = o; }
  void set_next_orientation(Orientation o) { m_next_orientation = o; }

  /*!
   */
  void compute_start_orientation(Vector2f & normal1, Vector2f & normal2)
  {
    Vector2f origin;
    m_start_orientation = normal1.orientation(origin, normal2);
  }
  void compute_end_orientation(Vector2f & normal1, Vector2f & normal2)
  {
    Vector2f origin;
    m_end_orientation = normal1.orientation(origin, normal2);
  }
  void compute_next_orientation(Vector2f & normal1, Vector2f & normal2)
  {
    Vector2f origin;
    m_next_orientation = normal1.orientation(origin, normal2);
  }
  
  /*!
   */
  unsigned int get_size() const { return m_range.get_size(); }
    
  /*!
   */
  void get_end_vertex(Vector2f & vertex)
  { get_vertex(m_range.get_end(), vertex); }

  /*!
   */
  void get_next_vertex_src(Vector2f & vertex)
  { get_vertex(m_range.get_next(), vertex); }

  /*!
   */
  void get_next_vertex_dst(Vector2f & vertex)
  {
    unsigned int id = (m_range.get_next() == (get_num_normals() - 1)) ?
      0 : m_range.get_next() + 1;
    get_vertex(id, vertex);
  }

  /*!
   */
  void get_start_normal(Vector2f & normal)
  { get_normal(m_range.get_start(), normal); }

  /*!
   */
  void get_end_normal(Vector2f & normal)
  { get_normal(m_range.get_end(), normal); }

  /*!
   */
  void get_next_normal(Vector2f & normal)
  { get_normal(m_range.get_next(), normal); }
  
  /*!
   */
  bool operator()(const Vector2f & dir);
};

/*!
 */
inline std::ostream & operator<<(std::ostream & os,
                                 const Circular_search & search)
{
  os << search.get_range();
  return os;
}  

#endif
