#ifndef CONF_SPACE_H
#define CONF_SPACE_H

#include "Agent.h"
#include "Frustum.h"
#include "Viewport.h"
#include "Reshape_viewport_events.h"
#include "Screen_poly.h"
#include "Robot_ref_sim.h"
#include "State.h"
#include "Mode.h"

class Trans_poly;
class Translate_event;
class Rotate_event;
class Extreme_vertices_event;
class Tick_event;
class Collision_test_event;
class Mode_event;
class Simulate_event;

class Conf_space : public Agent {
private:
  class Conf_space_state;

  typedef Screen_poly<Reshape_cs_viewport_event> Poly_cs_sim;

  enum States {
    IDLE_STATE = 0,
    FIND_EDGE_STATE,
    NUM_STATES
  };

  bool m_simulate;
  Mode m_mode;

  // Simulation stuff:
  unsigned int m_robot_extreme_vertex_id;
  unsigned int m_obstacle_extreme_vertex_id;

  int m_state_id;
  Conf_space_state * m_states[NUM_STATES];
  
  Frustum m_frustum;
  Viewport m_viewport;

  float m_scale;
  
  float m_near;
  float m_far;

  Robot_ref_sim m_robot_ref_sim;
  Poly_cs_sim m_obstacle_ref;
  
  float m_robot_radius;

  Trans_poly * m_rot_robot;
  Trans_poly * m_obstacle;

  Vector2f m_extreme_prev_vertex;
  Vector2f m_extreme_vertex;

  bool m_reshaped;

  void compute_collision_sequencial();
  void compute_collision_binary();
  void compute_collision();
  
protected:

public:
  /*!
   */
  Conf_space(void);
  
  /*!
   */
  virtual ~Conf_space(void);

  /*!
   */
  virtual void identify(void);

  /*!
   */
  virtual void handle(Mode_event * mode_event);

  /*!
   */
  virtual void handle(Simulate_event * event);
  
  /*!
   */
  virtual void handle(Reshape_ws_viewport_event * reshape_event);

  /*!
   */
  virtual void handle(Translate_event * translate_event);

  /*!
   */
  virtual void handle(Rotate_event * rotate_event);

  /*!
   */
  virtual void handle(Reset_event * event);

  /*!
   */
  virtual void handle(Collision_test_event * event);
  
  /*!
   */
  void draw(void);

  /*!
   */
  void init(void);

  /*!
   */
  void set_viewport(int vp_left, int vp_bottom, int vp_width, int vp_height);

  /*!
   */
  void set_rot_robot(Trans_poly & robot) { m_rot_robot = &robot; }

  /*!
   */
  void set_obstacle(Trans_poly & obstacle) { m_obstacle = &obstacle; }

  /*!
   */
  void compute_robot_ref(void);

  /*!
   */
  void compute_obstacle_ref(void);

  /*!
   */
  void set_state(int state_id);

private:
  /*! Generic Conf_space state:
   */
  class Conf_space_state : public State {
  private:
    Conf_space * m_conf_space;

  public:
    Conf_space_state(Conf_space * cs) : m_conf_space(cs) {}
    Conf_space * get_conf_space(void) const { return m_conf_space; }
    void change_state(int state_id);
    virtual void draw() {}
  };

  /*! Idle state (or start state)
   */
  class Idle_state : public Conf_space_state {
  public:
    Idle_state(Conf_space * cs) : Conf_space_state(cs) {}
    virtual void identify(void);
    virtual void draw();
    void on_entry(void);
    void on_exit(void);
    void handle(Extreme_vertices_event * event);
    void handle(Highlight_edge_event * event);
  };

  /*! Find edge state
   */
  class Find_edge_state : public Conf_space_state {
  private:
    Vector2f m_ref_dir;
    Orientation m_orientation;
    unsigned int m_robot_id;
    unsigned int m_obstacle_id;
    
  public:
    Find_edge_state(Conf_space * cs) : Conf_space_state(cs) {}
    virtual void identify(void);
    virtual void draw();
    void on_entry(void);
    void on_exit(void);
    void handle(Tick_event * Tick_event);
  };

  friend class Idle_state;
  friend class Find_edge_state;
};

#endif
