#ifndef DIAGRAM
#define DIAGRAM

#include "Arrow.h"
#include "State.h"
#include "Mode.h"

class Trans_poly;
class Trans_poly_sim_circular_search;

class Diagram : public Arrow {
private:
  class Diagram_state;

protected:
  enum States {
    IDLE_STATE = 0,
    SEARCH_STATE,
    NUM_STATES
  };

  float m_gap;
  
  int m_state_id;
  Diagram_state * m_states[NUM_STATES];
  
private:  
  Trans_poly * m_robot;
  Trans_poly * m_obstacle;
  
  float m_radius;
  int m_num_vertices;
  float m_line_width;
  float m_normal_line_width;

  void draw_poly(Trans_poly * poly);

private:
  /*! Generic Diagram state:
   */
  class Diagram_state : public State {
  protected:
    Diagram * m_diagram;
    int m_state_id;
    
  public:
    Diagram_state(Diagram * as, int state_id) :
      m_diagram(as), m_state_id(state_id)
    {}
    Diagram * get_diagram(void) const { return m_diagram; }
    void change_state(int state_id);
    virtual void draw() {}
  };

  /*! Idle state (or start state)
   */
  class Idle_state : public Diagram_state {
  public:
    Idle_state(Diagram * as, int id) : Diagram_state(as, id) {}
    virtual void identify(void);
    virtual void draw();
    void on_entry(void);
    void on_exit(void);
  };

  /*! Search state
   */
  class Search_state : public Diagram_state {
  private:
    
    Trans_poly_sim_circular_search * m_prev;
    Trans_poly_sim_circular_search * m_curr;
    Trans_poly_sim_circular_search * m_next;

    bool m_first;
    bool m_out_of_range;

    Vector2f m_end_mink_src;
    Vector2f m_end_mink_dst;

    Vector2f m_next_mink_src;
    Vector2f m_next_mink_dst;

    Orientation m_start_orientation;
    Orientation m_end_orientation;
    Orientation m_next_orientation;
    
    //! \todo move to Range
    // void draw_range(Range * range, float radius) const;
    void compute_mink();
    
  public:
    Search_state(Diagram * diag, int id);
    virtual ~Search_state();
    virtual void identify(void);
    virtual void draw();
    void on_entry(void);
    void on_exit(void);
    void handle(Tick_event * Tick_event);
  };
  
  friend class Idle_state;
  friend class Search_state;

protected:
  int m_robot_normal_id;
  int m_obstacle_normal_id;
  bool m_simulate;
  Mode m_mode;
  
public:
  /*!
   */
  Diagram(void);

  /*!
   */
  ~Diagram(void);

  /*!
   */
  void draw(void);

  /*!
   */
  void set_robot(Trans_poly * robot) { m_robot = robot; }

  /*!
   */
  void set_obstacle(Trans_poly * obstacle) { m_obstacle = obstacle; }

  /*!
   */
  void set_state(int state_id);

  /*!
   */
  void set_gap(float gap) { m_gap = gap; }
};

#endif
