#include <iostream>

#include "Diagram_sim.h"
#include "Reshape_viewport_event.h"
#include "Highlight_normals_event.h"
#include "Mode_event.h"
#include "Simulate_event.h"
#include "Translate_event.h"
#include "Rotate_event.h"
#include "Collision_test_event.h"

/*!
 */
Diagram_sim::Diagram_sim(void) :
  m_arrow_length(20), m_arrow_width(10), m_gap(10)
{
  Reshape_ds_viewport_event::doregister(this);
  Highlight_normals_event::doregister(this);
  Mode_event::doregister(this);
  Simulate_event::doregister(this);
  Collision_test_event::doregister(this);
  Translate_event::doregister(this);
  Rotate_event::doregister(this);
}

/*!
 */
Diagram_sim::~Diagram_sim(void)
{
  Collision_test_event::unregister(this);
  Translate_event::unregister(this);
  Rotate_event::unregister(this);
  Reshape_ds_viewport_event::unregister(this);
  Highlight_normals_event::unregister(this);
  Mode_event::unregister(this);
  Simulate_event::unregister(this);
}

/*!
 */
void Diagram_sim::identify(void)
{
  std::cout << "Agent: Diagram_sim" << std::endl;
}

/*!
 */
void Diagram_sim::handle(Reshape_ds_viewport_event * reshape_event)
{
#if 0
  if (s_verbose_level >= 1)
    std::cout << "Handling Reshape_diagram_viewport event: "
              << "Win scale: " << reshape_event->get_win_scale() << std::endl;
#endif
  float win_scale = reshape_event->get_win_scale();
  Diagram::set_arrow_length(m_arrow_length * win_scale);
  Diagram::set_arrow_width(m_arrow_width * win_scale);
  Diagram::set_gap(m_gap * win_scale);
}

/*!
 */
void Diagram_sim::handle(Highlight_normals_event * event)
{
  event->get(m_robot_normal_id, m_obstacle_normal_id);
}

/*!
 */
void Diagram_sim::init()
{
  set_state(IDLE_STATE);
}

/*!
 */
void Diagram_sim::handle(Mode_event * event)
{
  m_mode = event->get_mode();
}

/*!
 */
void Diagram_sim::handle(Simulate_event * event)
{
  m_simulate = event->get_simulate();
}

/*!
 */
void Diagram_sim::handle(Collision_test_event * event)
{
  if (m_simulate) {
    int next_state = (m_mode == BINARY) ? SEARCH_STATE : IDLE_STATE;
    m_states[m_state_id]->change_state(next_state);
  }
}

/*!
 */
void Diagram_sim::handle(Translate_event * event)
{
  m_states[m_state_id]->change_state(IDLE_STATE);
}

/*!
 */
void Diagram_sim::handle(Rotate_event * event)
{
  m_states[m_state_id]->change_state(IDLE_STATE);
}

