#if (defined _MSC_VER)
#include <windows.h>
#endif
#include <GL/gl.h>
#include <iostream>

#include "Diagram_space.h"
#include "Event_handler.h"
#include "Trans_poly.h"

/*!
 */
Diagram_space::Diagram_space(void) :
  m_near(10), m_far(-10), m_reshaped(false)
{
  Reshape_ds_viewport_event::doregister(this);
}

/*!
 */
Diagram_space::~Diagram_space(void)
{
  Reshape_ds_viewport_event::unregister(this);
}

/*!
 */
void Diagram_space::init(void)
{
  m_frustum.set_near_far(m_near, m_far);
  m_diagram_sim.init();
}
  
/*!
 */
void Diagram_space::draw(void)
{
  if (!m_reshaped) return;

  m_viewport();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  m_frustum.apply();
  glMatrixMode(GL_MODELVIEW);
  m_diagram_sim.draw();
}

/*!
 */
void Diagram_space::set_viewport(int vp_left, int vp_bottom,
                                 int vp_width, int vp_height)
{
  m_viewport.set(vp_left, vp_bottom, vp_width, vp_height);  

  double left = -1.0;
  double bottom = -1.0;
  double right = 1.0;
  double top = 1.0;
  double frustum_width = right - left;
  double frustum_height = top - bottom;
  if (frustum_width * vp_height > frustum_height * vp_width) {
    top = bottom + frustum_width * vp_height / vp_width;
  } else {
    right = left + frustum_height * vp_width / vp_height;
  }
  m_frustum.make_ortho(left, right, bottom, top);

  Reshape_ds_viewport_event * event = new Reshape_ds_viewport_event;
  event->set_win_scale(frustum_width / vp_width);
  Event_handler::issue(event);
}

/*!
 */
void Diagram_space::set_robot(Trans_poly & robot)
{
  m_diagram_sim.set_robot(&robot);
}

/*!
 */
void Diagram_space::set_obstacle(Trans_poly & obstacle)
{
  m_diagram_sim.set_obstacle(&obstacle);
}

/*!
 */
void Diagram_space::handle(Reshape_ds_viewport_event * reshape_event)
{
  m_reshaped = true;
}

/*!
 */
void Diagram_space::identify(void)
{ std::cout << "Diagram_space" << std::endl; }
