#if (defined _MSC_VER)
#include <windows.h>
#endif
#include <GL/gl.h>
#include <iostream>

#include "Normal_diagram_scene.h"
#include "Tick_event.h"
#include "Mode.h"
#include "Event_handler.h"
#include "Mode_event.h"
#include "Simulate_event.h"
#include "Collision_test_event.h"
#include "Sim.h"

int Normal_diagram_scene::s_verbose_level = 0;

/*!
 */
Normal_diagram_scene::Normal_diagram_scene(void) :
  m_simulate(false), m_mode(SEQUENCIAL),
  m_win_width(0), m_win_height(0),
  m_near(10), m_far(-10),
  m_line_width(2)
{
  m_conf_space.set_rot_robot(m_auxiliary_space.get_rot_robot());
  m_conf_space.set_obstacle(m_work_space.get_poly(1));

  m_auxiliary_space.set_robot(m_work_space.get_poly(0));
  m_auxiliary_space.set_obstacle(m_work_space.get_poly(1));
  
  m_diagram_space.set_robot(m_auxiliary_space.get_rot_robot());
  m_diagram_space.set_obstacle(m_work_space.get_poly(1));

  Tick_event::doregister(this);
}

/*!
 */
Normal_diagram_scene::~Normal_diagram_scene(void)
{
}

/*!
 */
void Normal_diagram_scene::init(void)
{
  glDisable(GL_LIGHTING);
  glDisable(GL_BLEND);
  glDisable(GL_CULL_FACE);
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_NORMALIZE);

  m_frustum.set_near_far(m_near, m_far);

  m_work_space.init();
  m_auxiliary_space.init();
  m_conf_space.init();
  m_diagram_space.init();
}

/*!
 */
void Normal_diagram_scene::identify(void)
{ std::cout << "Agent: Normal_diagram_scene" << std::endl; }

/*!
 */
void Normal_diagram_scene::handle(Tick_event * tick_event)
{
  if (s_verbose_level >= 1)
    std::cout << "Handling Tick event: "
              << "Sim time: " << tick_event->get_sim_time() << std::endl;
}

/*!
 */
void Normal_diagram_scene::draw_separators(void)
{
  m_viewport();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  m_frustum.apply();
  glMatrixMode(GL_MODELVIEW);

  int hwidth = m_win_width / 2;
  int hheight = m_win_height / 2;
  glLineWidth(m_line_width);
  glColor3f(0, 0, 0);
  glBegin(GL_LINES);
  glVertex2i(0, hheight);
  glVertex2i(m_win_width, hheight);
  glVertex2i(hwidth, 0);
  glVertex2i(hwidth, m_win_height);
  glEnd();
}

/*!
 */
void Normal_diagram_scene::draw(void)
{
  glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

  // Draw general gadgets:
  draw_separators();
    
  // Draw space specific:
  m_work_space.draw();
  m_auxiliary_space.draw();
  m_conf_space.draw();
  m_diagram_space.draw();
  glFlush();
}

/*!
 */
void Normal_diagram_scene::reshape(GLint width, GLint height)
{
  if (s_verbose_level >= 1)
    std::cout << "Normal_diagram_scene::reshape: "
              << "(" << width << "," << height << ")"
              << std::endl;
  m_win_width = width;
  m_win_height = height;
  int hwidth = m_win_width / 2;
  int hheight = m_win_height / 2;

  m_viewport.set(0, 0, width, height);
  m_frustum.make_ortho(-0.5f, width - 0.5f, -0.5f, height - 0.5f);
  
  m_work_space.set_viewport(0, 0, hwidth, hheight);
  m_auxiliary_space.set_viewport(hwidth, 0, hwidth, hheight);  
  m_conf_space.set_viewport(0, hheight, hwidth, hheight);
  m_diagram_space.set_viewport(hwidth, hheight, hwidth, hheight);  
}

void Normal_diagram_scene::keyboard(unsigned char c, int x, int y)
{
  switch (c) {
   case 'm':
    {
     m_mode = (m_mode == BINARY) ? SEQUENCIAL : (Mode) (m_mode + 1);
     Mode_event * event = new Mode_event;
     event->set_mode(m_mode);
     Event_handler::issue(event);
    }
    break;

   case 'e':
    {
     m_simulate = !m_simulate;
     Simulate_event * event = new Simulate_event;
     event->set_simulate(m_simulate);
     Event_handler::issue(event);
    }
    break;
    
   case 't':
    {
     Collision_test_event * test_event = new Collision_test_event;
     Event_handler::issue(test_event);
     Sim::start();
    }
    break;
  }
}
