#if (defined _MSC_VER)
#include "my_getopt.h"
#else
#include <stdlib.h>
#include <getopt.h>
#endif
#include "Parse_work_space_cl.h"

char * Parse_work_space_cl::s_option_str = "hv:i:";
char * Parse_work_space_cl::s_input_opts[] = {
  "file", "f",
  "vertices", "v",
  NULL
};

const char * Parse_work_space_cl::s_default_file_base_name = "poly";
const char * Parse_work_space_cl::s_default_file_ext_name = "txt";
const char * Parse_work_space_cl::s_default_dir_name = "data";

/*!
 */
Parse_work_space_cl::Parse_work_space_cl(void) :
  m_index(0), m_verbose_level(0)
{
  doregister(this);
  int i;
  for (i = 0; i < NUM_POLYS; i++) {
    m_file_name[i][0] = '\0';
  }
  m_shape_color[0][0] = 0.0f;
  m_shape_color[0][1] = 0.0f;
  m_shape_color[0][2] = 0.9f;
  m_shape_color[0][3] = 1.0f;

  m_shape_color[1][0] = 0.9f;
  m_shape_color[1][1] = 0.0f;
  m_shape_color[1][2] = 0.0f;
  m_shape_color[1][3] = 1.0f;

  m_num_vertices[0] = 16;
  m_num_vertices[1] = 16;
}

/*!
 */
void Parse_work_space_cl::print_help(void)
{
  std::cout << "Scene options:" << std::endl
            << "  -h\t\t\tprint this help message" << std::endl
            << "  -v <level>\t\tset verbose level" << std::endl
            << "  -i options\t\tset input options" << std::endl
            << "\t\tfile[=<file>\tset input file" << std::endl
            << "\t\tf[=<file>\tset input file" << std::endl
            << "\t\tvertices=<num>\tset number of vertices" << std::endl
            << "\t\tv=<num>\t\tset number of vertices" << std::endl;
}

/*!
 */
void Parse_work_space_cl::parse_input_options(char * options)
{
  char * value = 0;
  if (*options == '\0') return;
  while (*options != '\0') {
    switch(getsubopt(&options, s_input_opts, &value)) {
     case INPUT_FILE:
     case INPUT_F:
      if (m_index == NUM_POLYS) {
        std::cerr << "Exceeded number of polygons!" << std::endl;
        continue;
      }
      if (value) {
        strcpy(m_file_name[m_index++], value);
        break;
      }
      sprintf(m_file_name[m_index], "./%s/%s%d.%s",
              s_default_dir_name, s_default_file_base_name, m_index,
              s_default_file_ext_name); 
      m_index++;
      break;

     case NUM_VERTICES:
     case NUM_V:
      m_num_vertices[m_index++] = strtoul(value, 0, 0);
      break;
    }
  }
}

/*!
 */
int Parse_work_space_cl::operator()(void)
{
  optind = 1;
  opterr = 0;
  
  int c;
  while ((c = getopt(s_argc, s_argv, s_option_str)) != EOF) {
    switch (c) {
     case 'h': print_help(); m_done = true; break;
     case 'v': m_verbose_level = atoi(optarg); break;

     case 'i': parse_input_options(optarg); break;
     default: return 0;
    }
  }
  return 0;
}
