#ifndef SCREEN_POLY_H
#define SCREEN_POLY_H

#include <iostream>

#include "Agent.h"
#include "Trans_poly.h"
#include "Reshape_viewport_events.h"
#include "Translate_event.h"
#include "Rotate_event.h"

template <class Reshape_viewport_event_>
class Screen_poly : public Agent, public Trans_poly {
public:
  /*!
   */
  void identify(void) { std::cout << "Agent: Screen_poly" << std::endl; }
  
  /*!
   */
  virtual void handle(Reshape_viewport_event_ * reshape_event)
  {
    m_win_scale = reshape_event->get_win_scale();
    Trans_poly::set_normal_length(m_normal_length * m_win_scale);
    Trans_poly::set_arrow_length(m_arrow_length * m_win_scale);
    Trans_poly::set_arrow_width(m_arrow_width * m_win_scale);
    Trans_poly::set_ref_point_length(m_ref_point_length * m_win_scale);
    Trans_poly::set_vertex_radius(m_vertex_radius * m_win_scale);
  }

  virtual void handle(Translate_event * translate_event)
  {
    const Vector2f & vec = translate_event->get_vector();
    m_transform.set_translation(vec);
  }
  
  virtual void handle(Rotate_event * rotate_event)
  {
    float angle = rotate_event->get_angle();
    m_transform.set_angle(angle);
    Vector2f & center = m_poly.get_ref_point();
    m_transform.set_center(center);
  }
  
  /*! Parameterless constructor
   */
  Screen_poly(void) :
    m_ref_point_length(16), m_vertex_radius(3),
    m_normal_length(10.0f),
    m_arrow_length(20.0f), m_arrow_width(10.0f)
  {
    Reshape_viewport_event_::doregister(this);
  }

  /*! Copy constructor
   */
  Screen_poly(const Screen_poly & poly) :
    Poly(poly),
    m_win_scale(poly.m_win_scale),
    m_ref_point_length(poly.m_ref_point_length),
    m_normal_length(poly.m_normal_length),
    m_arrow_length(poly.m_arrow_length), m_arrow_width(poly.m_arrow_width)
  {
    Reshape_viewport_event_::doregister(this);
  }

  /*! Destructor
   */
  virtual ~Screen_poly(void)
  {
    Reshape_viewport_event_::unregister(this);
  }

private:
  // measured in pixels:
  float m_win_scale;
  
  float m_ref_point_length;
  float m_vertex_radius;
  float m_normal_length;
  float m_arrow_length;
  float m_arrow_width;
};

#endif
