#ifndef SIM_H
#define SIM_H

#if (defined _MSC_VER)
#include <windows.h>
#endif
// #include <conio.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <time.h>

#include "Scene.h"
#include "Event_handler.h"
#include "Agent.h"

class Stop_event;

class Sim : public Agent {
private:
  static void destroy(void);
  static void print_help(void);

private:
#define DEF_WIN_WIDTH   720
#define DEF_WIN_HEIGHT  576

  /*!
   */
  static void reshape(GLint width, GLint height);

  /*!
   */
  static void display(void);

  /*!
   */
  static void keyboard(unsigned char c, int x, int y);

  /*!
   */
  static void mouse(int button, int state, int x, int y);

  /*!
   */
  static void motion(int x, int y);

  /*!
   */
  static void special(int key, int x, int y);

  /*!
   */
  static void visible(int vis);

  /*!
   */
  static void idle(void);

  /*!
   */
  void init_gfx(void);
  
public:
  /*!
   */
  Sim(Scene * scene, const char * scene_name);

  /*!
   */
  virtual ~Sim();
  
  /*!
   */
  void init(int * argcp, char ** argv);

  /*!
   */
  static void set_verbose_level(int verbose_level);

  /*!
   */
  virtual void handle(Stop_event * event);

  /*!
   */
  virtual void identify();

  /*!
   */
  void operator() ();

  /*!
   */
  static void set_required_tick_duration(clock_t duration);

  /*!
   */
  static void stop(void);

  /*!
   */
  static void start(void);

  /*!
   */
  static void suspend(void);

  /*!
   */
  static void resume(void);

  /*!
   */
  static void next(void);
  
private:
  static bool s_do_double_buffer;  
  static Scene * s_scene;
  static Event_handler s_event_handler;

  static clock_t s_start_tick_time;
  static clock_t s_sim_time;
  static clock_t s_est_tick_duration;
  static clock_t s_required_tick_duration;

  static int s_verbose_level;
  
  static int s_win_width;
  static int s_win_height;

  static bool m_simulating;

  const char * m_scene_name;
  bool m_full_screen;
  bool m_interactive;
};

/*!
 */
inline void Sim::operator() () { glutMainLoop(); }

#endif
