#if (defined _MSC_VER)
#include <windows.h>
#endif
#include <GL/gl.h>
#include <iostream>

#include "Trans_poly.h"
#include "Orientation.h"
#include "Trans_poly_circular_search.h"

/*!
 */
void Trans_poly::transform()
{
  const Matrix3f & matrix = m_transform.get_matrix();
  float m[16];
  m[0] = matrix.get(0,0);
  m[1] = matrix.get(0,1);
  m[2] = 0.0;
  m[3] = matrix.get(0,2);

  m[4] = matrix.get(1,0);
  m[5] = matrix.get(1,1);
  m[6] = 0.0;
  m[7] = matrix.get(1,2);

  m[8] = 0.0;
  m[9] = 0.0;
  m[10] = 1.0;
  m[11] = 0.0;

  m[12] = matrix.get(2,0);
  m[13] = matrix.get(2,1);
  m[14] = 0.0;
  m[15] = matrix.get(2,2);

  glLoadMatrixf(m);
}

/*!
 */
void Trans_poly::draw(void)
{
  glPushMatrix();
  transform();
  m_poly.draw();
  glPopMatrix();
}

/*!
 */
void Trans_poly::draw_shape(bool wire_frame)
{
  glPushMatrix();
  transform();
  m_poly.draw_shape(wire_frame);
  glPopMatrix();
}

/*!
 */
void Trans_poly::draw_normals(void)
{
  glPushMatrix();
  transform();
  m_poly.draw_normals();
  glPopMatrix();
}

/*!
 */
void Trans_poly::draw_ref_point(void)
{
  glPushMatrix();
  transform();
  m_poly.draw_ref_point();
  glPopMatrix();
}

/*!
 */
void Trans_poly::draw_normal(int id)
{
  glPushMatrix();
  transform();
  m_poly.draw_normal(id);
  glPopMatrix();
}

/*!
 */
void Trans_poly::draw_vertices(void)
{
  glPushMatrix();
  transform();
  m_poly.draw_vertices();
  glPopMatrix();
}

/*!
 */
void Trans_poly::draw_vertex(int id)
{
  glPushMatrix();
  transform();
  m_poly.draw_vertex(id);
  glPopMatrix();
}

/*!
 */
void Trans_poly::flatten(void)
{
  m_poly.transform(m_transform);
  m_transform.init();
}

/*!
 */
void Trans_poly::compute_opposite(Trans_poly * tp)
{
  m_poly.compute_opposite(tp->get_poly(), tp->get_transform());
}

/*!
 */
void Trans_poly::compute_minkowski(Trans_poly * trans_poly1,
                                   Trans_poly * trans_poly2)
{
  Poly tmp1(trans_poly1->get_poly(), trans_poly1->get_transform());
  Poly tmp2(trans_poly2->get_poly(), trans_poly2->get_transform());
  m_poly.compute_minkowski(tmp1, tmp2);
}

/*!
 */
void Trans_poly::get_normal(int index, Vector2f & normal)
{
  const Vector2f & vec = m_poly.get_normal(index);
  const Matrix3f & matrix = m_transform.get_matrix();
  normal.xform_vec(vec, matrix);  
}
  
/*!
 */
void Trans_poly::get_vertex(int index, Vector2f & vertex)
{
  const Vector2f & vec = m_poly.get_vertex(index);
  const Matrix3f & matrix = m_transform.get_matrix();
  vertex.xform_pt(vec, matrix);  
}
  
/*!
 */
void Trans_poly::get_ref_point(Vector2f & pt)
{
  const Vector2f & q = m_poly.get_ref_point();
  const Matrix3f & matrix = m_transform.get_matrix();
  pt.xform_pt(q, matrix);  
}

/*!
 * \pre dir is normalized
 */
unsigned int Trans_poly::find_extreme_vertex(Vector2f & dir)
{
#if 0
  Poly tmp(m_poly, m_transform);
  return tmp.find_extreme_vertex(dir);
#else
  if (get_num_normals() == 0) return 0;

  Trans_poly_circular_search search(this);
  search(dir);
  return search.get_end();
#endif
}
