#ifndef TRANS_POLY_H
#define TRANS_POLY_H

#include "Vector2f.h"
#include "Vector4f.h"
#include "Poly.h"
#include "Transform_2.h"

class BBox_2f;

class Trans_poly {
private:
  void transform();

protected:
  Poly m_poly;
  Transform_2 m_transform;

public:
  Poly & get_poly(void) { return m_poly; }
  Transform_2 & get_transform(void) { return m_transform; }
  
  void init_regular(int num, float radius) { m_poly.init_regular(num, radius); }
  bool read(const char * file_name) { return m_poly.read(file_name); }
  
  void draw(void);
  void draw_shape(bool wire_frame = false);
  void draw_normals(void);
  void draw_normal(int id);
  void draw_ref_point(void);
  void draw_vertices(void);
  void draw_vertex(int id);

  void render_shape(bool wire_frame = false) const { m_poly.render_shape(); }
  
  const RGBA_color & get_shape_color(void) const
  { return m_poly.get_shape_color(); }
  void set_shape_color(const RGBA_color & color)
  { m_poly.set_shape_color(color); }
  void set_shape_alpha(float alpha)
  { m_poly.set_shape_alpha(alpha); }  

  void set_normal_color(const RGBA_color & color)
  { m_poly.set_normal_color(color); }

  void set_alpha(float alpha) { m_poly.set_alpha(alpha); }
  
  void compute_opposite(Trans_poly * trans_poly);
  void compute_normals(void) { m_poly.compute_normals(); }
  void compute_bbox(void) { m_poly.compute_bbox(); }
  void compute_ref_point(void) { m_poly.compute_ref_point(); }
  void compute_minkowski(Trans_poly * trans_poly1, Trans_poly * trans_poly2);

  void set_normal_length(float length) { m_poly.set_normal_length(length); }
  void set_arrow_length(float length) { m_poly.set_arrow_length(length); }
  void set_arrow_width(float width) { m_poly.set_arrow_width(width); }
  void set_ref_point_length(float length)
  { m_poly.set_ref_point_length(length); }
  void set_vertex_radius(float radius) { m_poly.set_vertex_radius(radius); }

  void translate(float x, float y) { m_poly.translate(x, y); }
  void rotate(float angle) { m_poly.rotate(angle); }
  void flatten(void);
  
  BBox_2f & get_bbox(void) { return m_poly.get_bbox(); }
  void get_ref_point(Vector2f & pt);
  float get_radius(void) { return m_poly.get_radius(); }
  
  unsigned int get_num_normals() { return m_poly.get_num_normals(); }
  void get_normal(int index, Vector2f & normal);
  void get_vertex(int index, Vector2f & vertex);

  unsigned int find_extreme_vertex(Vector2f & dir);
};

#endif
