#ifndef TRANS_POLY_CIRCULAR_SEARCH
#define TRANS_POLY_CIRCULAR_SEARCH

#include <iostream>
#include "Circular_search.h"
#include "Trans_poly.h"
#include "Arrow.h"

class Trans_poly_circular_search : public Circular_search {
private:
  Trans_poly * m_poly;
  unsigned int m_num_vertices;
  float m_line_width;
  Arrow m_arrow;

protected:
  void set_arrow_length(float length) { m_arrow.set_arrow_length(length); } 
  void set_arrow_width(float width) { m_arrow.set_arrow_width(width); }
  
public:
  /*!
   */
  Trans_poly_circular_search() :
    m_poly(0),
    m_num_vertices(32),
    m_line_width(1)
  {
  }

  /*!
   */
  Trans_poly_circular_search(Trans_poly * poly) :
    m_poly(poly),
    m_num_vertices(32),
    m_line_width(1)
  {
    // Initialize range:
    m_range.set_num(get_num_normals());
  }

  /*!
   */
  Trans_poly_circular_search(const Trans_poly_circular_search & clone) :
    Circular_search(clone),
    m_poly(clone.m_poly),
    m_num_vertices(32),
    m_line_width(1)
  {
  }

  /*!
   */
  Trans_poly_circular_search &
  operator=(const Trans_poly_circular_search & clone)
  {
#if 1
    m_start_orientation = clone.m_start_orientation;
    m_end_orientation = clone.m_end_orientation;
    m_next_orientation = clone.m_next_orientation;
    m_range = clone.m_range;
#else
    Circular_search(clone);
#endif
    m_poly = clone.m_poly;
    m_num_vertices = clone.m_num_vertices;
    m_line_width = clone.m_line_width;
    return *this;
  }

  /*!
   */
  const Trans_poly * get_poly() const { return m_poly; }
  
  /*!
   */
  void set_poly(Trans_poly * poly)
  {
    m_poly = poly;
    // Initialize range:
    m_range.set_num(poly->get_num_normals());    
  }
  
  /*!
   */
  virtual void get_normal(unsigned int id, Vector2f & normal) const
  {
    m_poly->get_normal(id, normal);
  }

  /*!
   */
  virtual void get_vertex(unsigned int id, Vector2f & vertex) const
  {
    m_poly->get_vertex(id, vertex);
  }
  
  /*!
   */
  virtual unsigned int get_num_normals() const
  {
    return m_poly->get_num_normals();
  }

  /*!
   */
  void draw(float radius);
};

#endif
