#include <iostream>

#include "Transform_2.h"



Vector2f Transform_2::m_def_translation(0, 0);

float Transform_2::m_def_angle = 0;

Vector2f Transform_2::m_def_scale(1, 1);

Vector2f Transform_2::m_def_center(0, 0);

Vector2f Transform_2::m_def_bbox_center(0, 0);

Vector2f Transform_2::m_def_bbox_size(-1, -1);



/*!

 */

Transform_2::Transform_2() :

  m_translation(m_def_translation),

  m_angle(m_def_angle), 

  m_scale(m_def_scale),

  m_center(m_def_center),

  m_dirty_matrix(false), 

  m_dirty_inverse(false), 

  m_dirty_parts(false)

{

}



/*!

 */

Transform_2::~Transform_2()

{

}



/*!

 */

void Transform_2::set_matrix(const Matrix3f & matrix)

{

  m_matrix.set(matrix);

  m_dirty_parts = true;

  m_dirty_matrix = false;

  m_dirty_inverse = true;

}



/*!

 */

const Matrix3f & Transform_2::get_matrix()

{

  if (m_dirty_matrix) update_matrix();

  return m_matrix;

}



/*! Build the matrix out of the individual Transform_2s

 */

void Transform_2::update_matrix()

{

  float angle = get_angle();

  float mySin = sinf(angle);

  float myCos = cosf(angle);



  // TBD: really inefficient!!!

  

  Matrix3f t1;

  Matrix3f t2;

  Matrix3f t3;

  

  Vector2f tmp;

  get_center(tmp);

  tmp[0] = -tmp[0];

  tmp[1] = -tmp[1];

  t1.set_row(2, tmp);



  t2[0][0] = myCos * m_scale[0];

  t2[0][1] = mySin * m_scale[0];

  t2[1][0] = -mySin * m_scale[1];

  t2[1][1] = myCos * m_scale[1];



  get_center(tmp);

  tmp.add(m_translation);

  t3.set_row(2, tmp);



  m_matrix.mult(t3, t2);

  m_matrix.mult(m_matrix, t1);

  

  m_dirty_matrix = false;

}



/*! TBD: worng!

 * Extracts the individual Transform_2 from the matrix.

 */

void Transform_2::update_parts()

{

  m_translation.set(m_matrix[2][0], m_matrix[2][1]);

  m_angle = 0.0f;

  m_scale.set(1.0f, 1.0f);

  m_dirty_parts = false;

}



/*! TBD: worng!

 * Extracts the individual Transform_2 from the matrix.

 */

void Transform_2::init()

{

  m_translation.set(0.0f, 0.0f);

  m_angle = 0.0f;

  m_scale.set(1.0f, 1.0f);

  m_dirty_parts = false;

  m_dirty_matrix = true;

}



/*!

 */

void Transform_2::get_translation(float * v0, float * v1)

{

  if (m_dirty_parts) {

    *v0 = m_matrix[2][0];

    *v1 = m_matrix[2][1];

    return;

  }

  m_translation.get(v0, v1);

}



/*!

 */

void Transform_2::set_translation(float v0, float v1)

{

  if (m_dirty_parts) update_parts();

  m_translation.set(v0, v1);

  m_dirty_matrix = true;

  // m_dirty_inverse = true;

}



/*!

 */

void Transform_2::set_angle(float angle)

{

  if (m_dirty_parts) update_parts();

  m_angle = angle;

  m_dirty_matrix = true;

  m_dirty_inverse = true;

}



/*!

 */

void Transform_2::set_scale(float s0, float s1)

{

  if (m_dirty_parts) update_parts();

  m_scale[0] = s0;

  m_scale[1] = s1;

  m_dirty_matrix = true;

  m_dirty_inverse = true;

}



/*!

 */

void Transform_2::set_translation(const Vector2f & translation)

{

  set_translation(translation[0], translation[1]);

}



/*!

 */

void Transform_2::get_translation(Vector2f & translation)

{

  get_translation(&translation[0], &translation[1]);

}



/*!

 */

float Transform_2::get_angle()

{

  if (m_dirty_parts) update_parts();

  return m_angle;

}



/*!

 */

void Transform_2::get_scale(float * v0, float * v1)

{

  if (m_dirty_parts) update_parts();

  *v0 = m_scale[0];

  *v1 = m_scale[1];

}



/*!

 */

void Transform_2::set_scale(const Vector2f & scale)

{

    set_scale(scale[0], scale[1]);

}



/*!

 */

void Transform_2::get_scale(Vector2f & scale)

{

  get_scale(&scale[0], &scale[1]);

}



/*!

 */

void Transform_2::set_center(float v0, float v1)

{

  if (m_dirty_parts) update_parts();

  m_center[0] = v0;

  m_center[1] = v1;

  m_dirty_matrix = true;

  m_dirty_inverse = true;

}



/*!

 */

void Transform_2::get_center(float * v0, float * v1)

{

  if (m_dirty_parts) update_parts();

  *v0 = m_center[0];

  *v1 = m_center[1];

}



/*!

 */

void Transform_2::set_center(const Vector2f & center)

{

  set_center(center[0], center[1]);

}



/*!

 */

void Transform_2::get_center(Vector2f & center)

{

  get_center(&center[0], &center[1]);

}



/*!

 */

Transform_2::Transform_2(const Transform_2 & xform)

{

  *this = xform;

}



/*!

 */

const Transform_2 & Transform_2::operator=(const Transform_2 & xform)

{

  if (&xform != this)

  {

    m_dirty_matrix = xform.m_dirty_matrix;

    m_dirty_inverse = xform.m_dirty_inverse;

    m_dirty_parts = xform.m_dirty_parts;

    if (!m_dirty_matrix) m_matrix.set(xform.m_matrix);

    if (!m_dirty_parts) {

      m_translation.set(xform.m_translation);

      m_angle = xform.m_angle;

      m_scale.set(xform.m_scale);

      m_center.set(xform.m_center);

    }

    if (!m_dirty_inverse) m_inverse_matrix.set(xform.m_inverse_matrix);

  }

  return *this;

}



