#ifndef TRANSFORM_H
#define TRANSFORM_H

#include "Vector2f.h"
#include "Matrix3f.h"

class Transform_2 {
private:
  /*! the translation vector */
  Vector2f m_translation;

  /*! the rotation angle */
  float m_angle;

  /*! the scale vector */
  Vector2f m_scale;

  /*! the center of the transform */
  Vector2f m_center;

  /*! matrix representation of the transformation */
  Matrix3f m_matrix;

  /*! matrix transformation of the inverse matrix */
  Matrix3f m_inverse_matrix;

  /*! this is true when the matrix is not up-to-date */
  bool m_dirty_matrix;

  bool m_dirty_inverse;
  bool m_dirty_parts;

  // default values 

  /*! the default translation vector */
  static Vector2f m_def_translation;

  /*! the default rotation vector */
  static float m_def_angle;

  /*! the default scale vector */
  static Vector2f m_def_scale;

  /*! the default center of the transform */
  static Vector2f m_def_center;
  
  /*! the default bbox  center */
  static Vector2f m_def_bbox_center;

  /*! the default bbox size */
  static Vector2f m_def_bbox_size;

  void update_parts();
  void calc_matrix();
  void update_matrix();
  void clean_parts();
  
public:
  Transform_2();
  Transform_2(const Transform_2 & xform);
  ~Transform_2();

  void init();
  
  void set_translation(const Vector2f & translation);
  void get_translation(Vector2f & translation);

  void set_angle(float angle);
  float get_angle();

  void set_scale(const Vector2f & scale);
  void get_scale(Vector2f & scale);

  void set_center(const Vector2f & center);
  void get_center(Vector2f & center);

  void set_matrix(const Matrix3f & matrix);
  const Matrix3f & get_matrix(void);

  void set_translation(float v0, float v1);
  void get_translation(float * v0, float * v1);

  void set_scale(float v0, float v1);
  void get_scale(float * v0, float * v1);

  void set_center(float v0, float v1);
  void get_center(float * v0, float * v1);

  const Transform_2 & operator=(const Transform_2 & xform);
};

#endif
