#ifndef WORK_SPACE_H

#define WORK_SPACE_H



#include "Agent.h"

#include "Screen_poly.h"

#include "Frustum.h"

#include "Viewport.h"

#include "Parse_work_space_cl.h"

#include "Reshape_viewport_events.h"

#include "State.h"



class Mouse_event;

class Mouse_ws_event;

class Motion_event;



class Work_space : public Agent {

private:

  class Work_space_state;



  typedef Screen_poly<Reshape_ws_viewport_event>   Poly_ws_sim;

  

  static unsigned char * s_pixels;

  

  Parse_work_space_cl m_parse;

  Poly_ws_sim m_poly[NUM_POLYS];

  Frustum m_frustum;

  Viewport m_viewport;



  enum States {

    START_STATE = 0,

    LEFT_STATE,

    RIGHT_STATE,

    LEFT_RIGHT_STATE,

    NUM_STATES

  };

  

  Work_space_state * m_states[NUM_STATES];



  unsigned int m_num_polys;

  

  float m_near;

  float m_far;

  

  int m_num_vertices;

  float m_radius;



  int m_state_id;



  unsigned int m_poly_id;

  int m_x;

  int m_y;



  void reset();

  

public:

  virtual void identify(void);



  /*!

   */

  Work_space(void);



  /*!

   */

  virtual ~Work_space(void);

  

  /*!

   */

  virtual void handle(Mouse_event * mouse_event);



  /*!

   */

  void draw(void);



  /*!

   */

  void init(void);



  /*!

   */

  void set_viewport(int vp_left, int vp_bottom, int vp_width, int vp_height);



  /*!

   */

  const Viewport & get_viewport(void) const { return m_viewport; }



  /*!

   */

  Frustum & get_frustum(void) { return m_frustum; }

  

  /*!

   */

  Poly_ws_sim & get_poly(int i) { return m_poly[i]; }

  Poly_ws_sim & get_poly(void) { return m_poly[m_poly_id]; }



  /*!

   */

  void set_state(int state_id);



  void set_poly_id(unsigned int id) { m_poly_id = id; }

  unsigned int get_poly_id(void) const { return m_poly_id; }



  void set_x(int x) { m_x = x; }

  int get_x(void) const { return m_x; }



  void set_y(int y) { m_y = y; }

  int get_y(void) const { return m_y; }



  unsigned int get_num_polys() const { return m_num_polys; }

  unsigned char * get_pixels() const { return s_pixels; }

  

private:

  /*! Generic Work_space state:

   */

  class Work_space_state : public State {

  private:

    Work_space * m_work_space;



  public:

    /*! Constructor

     */

    Work_space_state(Work_space * ws) : m_work_space(ws) {}



    /*!

     */

    Work_space * get_work_space(void) const { return m_work_space; }

    

    /*!

     */

    void change_state(int state_id);

  };



  /*! Start state (or idle state)

   */

  class Start_state : public Work_space_state {

  public:

    /*!

     */

    Start_state(Work_space * ws) : Work_space_state(ws) {}



    /*!

     */

    virtual void handle(Mouse_ws_event * mouse_event);



    /*!

     */

    virtual void identify(void);



    /*!

     */

    void on_entry(void);



    /*!

     */

    void on_exit(void);

  };



  /*! Left state

   */

  class Left_state : public Work_space_state {

  public:

    /*!

     */

    Left_state(Work_space * ws) : Work_space_state(ws) {}



    /*!

     */

    virtual void handle(Mouse_ws_event * mouse_event);



    /*!

     */

    virtual void handle(Motion_event * event);



    /*!

     */

    virtual void identify(void);



    /*!

     */

    void on_entry(void);



    /*!

     */

    void on_exit(void);

  };  



  /*! Right state

   */

  class Right_state : public Work_space_state {

  public:

    /*!

     */

    Right_state(Work_space * ws) : Work_space_state(ws) {}



    /*!

     */

    virtual void handle(Mouse_ws_event * mouse_event);



    /*!

     */

    virtual void handle(Motion_event * event);



    /*!

     */

    virtual void identify(void);



    /*!

     */

    void on_entry(void);



    /*!

     */

    void on_exit(void);

  };  



  /*!Left & right state

   */

  class Left_right_state : public Work_space_state {

  public:

    /*!

     */

    Left_right_state(Work_space * ws) : Work_space_state(ws) {}



    /*!

     */

    virtual void handle(Mouse_ws_event * mouse_event);



    /*!

     */

    virtual void handle(Motion_event * event);



    /*!

     */

    virtual void identify(void);



    /*!

     */

    void on_entry(void);



    /*!

     */

    void on_exit(void);

  };  



  friend class Start_state;

  friend class Left_state;

  friend class Right_state;

  friend class Left_right_state;

};



#endif

