#ifndef PLANE_H

#define PLANE_H



#include "Vector3f.h"



class EBoxBound;

class ESphereBound;

class ESeg;

class Matrix4f;



class Plane {

public:

  void set_normal(float x, float y, float z) { m_normal.set(x,y,z); }

  void set_normal(const Vector3f & n) { m_normal.set(n); }

  void get_normal(Vector3f & n) const { m_normal.get(n); }



  void set_offset(float o) { m_offset = o; }

  float get_offset() const { return m_offset; }



public:

  /*! Constructs a plane from the three points <p1> <p2>, and <p3>,

   * assuming they form a counter-clockwise triangle viewed from the front.

   */

  void make_pts(const Vector3f & p1, const Vector3f & p2, const Vector3f & p3);



  /*! Stores the normal-point representation of the plane in norm and point.

   */

  void make_norm_pt(const Vector3f & norm, const Vector3f & pos);



  void copy(const Plane & p)

  { m_normal.set(p.m_normal); m_offset = p.m_offset; }



public:

#if 0

  // (See Bound::contains().)

  unsigned int contains(const EBoxBound * box) const;

  unsigned int contains(const ESphereBound * sph) const;

  unsigned int contains(const Vector3f & pt) const;



  // (See Bound::isect().)

  unsigned int isect(const Seg * seg, float * d) const;

  unsigned int isect(const Seg * seg, float * d1, float * d2) const;

#endif

  

  // Stores in <dst> the point on this plane closest to <pt>.

  void closest_pt_on(const Vector3f & pt, Vector3f & dst) const;



public:

  // Stores in this half space the result of transforming <pln> by the

  // ortho transform <m>.

    void ortho_xform(const Plane * pln, const Matrix4f & m);



public:

  /*! Normal to the plane */

  Vector3f m_normal;           



  /*! Offset in the plane equation. pt dot normal = offset */

  float m_offset;

};



#if 0

/*!

 */

inline unsigned int Plane::contains(const Vector3f & pt) const

{

  // Test if point pt on positive side of plane p

  return (pt.dot(m_normal) <= m_offset);

}

#endif



#endif

