#include "Vector2f.h"

#include "Matrix3f.h"



/*!

 */

float Vector2f::get_max_comp()

{

  float max = ( m_vector[0] > m_vector[1] ? m_vector[0] : m_vector[1] );

  return max;

}



/*!

 */

float Vector2f::normalize()

{

  double l2 = m_vector[0] * m_vector[0] + m_vector[1] * m_vector[1];

  float length = (float) sqrt(l2);

  if (length == 0.0f) {

    return length;

  }



  float length_reciprocal = 1.0f / length;

  m_vector[0] *= length_reciprocal;

  m_vector[1] *= length_reciprocal;

  return length;

}



/*! iteration of precision

 */

void Vector2f::normalize1()

{

  float length_reciprocal = length_reciprocal1();

  m_vector[0] *= length_reciprocal;

  m_vector[1] *= length_reciprocal;

}



/*! Sets this vector to be v, thought of as a row vector, times the 3X3

 * submatrix formed by the first 3 rows and first three columns of m.

 */

void Vector2f::xform_vec(const Vector2f & v, const Matrix3f & m)

{

  Vector2f t;

  t.set(v);

  m_vector[0] = t[0]*m[0][0] + t[1]*m[1][0];

  m_vector[1] = t[0]*m[0][1] + t[1]*m[1][1];

}



/*! Sets this vector to be the first three components of (v,1) * m where v

 * is treated as a row vector.

 */

void Vector2f::xform_pt(const Vector2f & v, const Matrix3f & m)

{

  Vector2f t;

  t.set(v);

  m_vector[0] = t[0]*m[0][0] + t[1]*m[1][0] + m[2][0];

  m_vector[1] = t[0]*m[0][1] + t[1]*m[1][1] + m[2][1];

}



//////////////////// NOT USED /////////////////////////

#ifdef ENB_NO_PLUGIN



/** Clamp */

void Vector2f::clamp(const Vector2f & v,

                      const Vector2f & min, const Vector2f & max)

{

  m_vector[0] = (v[0] < min[0]) ? min[0] : (max[0] < v[0]) ? max[0] : v[0];

  m_vector[1] = (v[1] < min[1]) ? min[1] : (max[1] < v[1]) ? max[1] : v[1];

}



/*! Sets this vector to be the inhomogeneous coordinates (that, divided by w) 

 * of (v,1) * m where v is treated as a row vector. 

 */

void Vector2f::full_xform_pt(const Vector2f & v, const Matrix3f & m)

{

  Vector2f t;

  t.set(v);

  float w = t[0]*m[0][3] + t[1]*m[1][3] + m[2][2];

  m_vector[0] = (t[0]*m[0][0] + t[1]*m[1][0] + m[2][0]) / w;

  m_vector[1] = (t[0]*m[0][1] + t[1]*m[1][1] + m[2][1]) / w;

}



#endif

