#include "Matrix3f.h"

#include "Vector3f.h"



/*! Sets this vector to be v * m where v is treated as a row vector.

 */

void Vector3f::xform(const Vector3f & v, const Matrix3f & m)

{

  Vector3f t;

  t.set(v);

  m_vector[0] = t[0]*m[0][0] + t[1]*m[1][0] + t[2]*m[2][0];

  m_vector[1] = t[0]*m[0][1] + t[1]*m[1][1] + t[2]*m[2][1];

  m_vector[2] = t[0]*m[0][2] + t[1]*m[1][2] + t[2]*m[2][2];

}



/*!  Sets this vector to be the cross product of v1 and v2.

 */

void Vector3f::cross(const Vector3f & v1, const Vector3f & v2)

{

  m_vector[0] = v1[1] * v2[2] - v1[2] * v2[1];

  m_vector[1] = v1[2] * v2[0] - v1[0] * v2[2];

  m_vector[2] = v1[0] * v2[1] - v1[1] * v2[0];

}



/*!

 */

float Vector3f::normalize()

{

  float len = length();

  if (len == 0.0f) return len;

  float length_reciprocal = 1.0f / len;

  m_vector[0] *= length_reciprocal;

  m_vector[1] *= length_reciprocal;

  m_vector[2] *= length_reciprocal;

  return len;

}

