#ifndef FRUSTUM_H

#define FRUSTUM_H



#include "Plane.h"



class Frustum

{

public:

  // Constructors, destructors

  Frustum();

  virtual ~Frustum();



public:

  // Enums

  enum Aspect_mode {

    CALC_NONE,

    CALC_HORIZ,

    CALC_VERT

  };



  enum Frustum_type {

    SIMPLE,

    ORTHOGONAL,

    PERSPECTIVE

  };



  enum Frust_planes {

    LEFT_PLANE=0,

    RIGHT_PLANE,

    NEAR_PLANE,

    FAR_PLANE,

    BOTTOM_PLANE,

    TOP_PLANE,

    NUM_PLANES

  };



  enum Frust_corners {

    NEAR_LL=0,

    NEAR_LR,

    NEAR_UL,

    NEAR_UR,

    FAR_LL,

    FAR_LR,

    FAR_UL,

    FAR_UR,

    NUM_CORNERS

  };

    

public:

  virtual void copy(const Frustum * src);



public:

  Frustum::Frustum_type get_frustum_type() const { return m_frustum_type; }



  void set_near(double near_dist);

  void set_far(double far_dist);



  void set_near_far(double near_dist, double far_dist);

  void get_near_far(double & near_dist, double & far_dist);

    

  void set_fov(double fov);

  double get_fov();

  void set_horiz_fov(double horiz_fov);

  void set_vert_fov(double vert_fov);

  void set_right(double right);

  void set_left(double left);

  void set_top(double top);

  void set_bottom(double bottom);



  void get_near(Vector3f & ll, Vector3f & lr, Vector3f & ul, Vector3f & ur);

  void get_far(Vector3f & ll, Vector3f & lr, Vector3f & ul, Vector3f & ur);



  void get_diag_corners(double & left, double & right,

                        double & bottom, double & top,

                        double & near_dist, double & far_dist);



  void set_aspect_mode(Frustum::Aspect_mode mode);

  Frustum::Aspect_mode get_aspect_mode() const;



  void set_aspect_ratio(double ratio);

  double get_aspect_ratio() { return m_aspect_ratio; }



  void get_GL_proj_mat(Matrix4f & mat);



  Plane * get_facets();



public:

  void make_persp(double left, double right, double bot, double top);

  void make_ortho(double left, double right, double bot, double top);



#if 0

  unsigned int contains(const Vector3f & pt) const;

  unsigned int contains(const ESphereBound * sphere) const;

  unsigned int contains(const EBoxBound * box) const;

#endif

  

  void apply();



protected:

  // void calc(Frustum::Aspect_mode aspect_mode);

  void update_corners();

  void update_planes();  



private:

  Frustum::Frustum_type m_frustum_type;

  Frustum::Aspect_mode m_aspect_mode;

  double m_near_dist;

  double m_far_dist;

  double m_left;

  double m_right;

  double m_top;

  double m_bottom;

  double m_vert_fov;

  double m_horiz_fov;

  double m_center_x;

  double m_center_y;

  double m_aspect_ratio;

  Vector3f m_corners[Frustum::NUM_CORNERS];

  Plane m_facets[Frustum::NUM_PLANES];



  bool m_dirty_corners;

  bool m_dirty_planes;

};



#endif

